/* This file is Copyright 2001 Level Control Systems.  See the included LICENSE.txt file for details. */

#ifndef THREAD_SAFE_OBJECT_POOL_H
#define THREAD_SAFE_OBJECT_POOL_H

#include <support/Locker.h>
#include "pool/ObjectPool.h"

/** 
 *  Same as your average ObjectPool, but uses a BLocker to serialize operations,
 *  so that it may be safely shared between threads.  Only compiles under BeOS!
 */
template <class Object> class ThreadSafeObjectPool : public ObjectPool<Object>
{
public:
   /**
    *  Constructor.
    *  @param maxPoolSize the maximum number of recycled objects that may be kept around for future reuse at any one time.
    *  @param recycleCallback optional callback function to be called whenever an object is returned to the pool.
    *  @param recycleData user value to be passed in to the recycleCallback function.
    *  @param initCallback optional callback function to be called whenever an object is taken out of the pool.
    *  @param initCallbackData user value to be passed in to the initCallback function.
    */
   ThreadSafeObjectPool(uint32 maxPoolSize=100,
              ObjectCallback recycleCallback = NULL, void * recycleData = NULL,
              ObjectCallback initCallback = NULL, void * initData = NULL) :
      ObjectPool<Object>(maxPoolSize, recycleCallback, recycleData, initCallback, initData) {/* empty */}

   /** Virtual Destructor */
   virtual ~ThreadSafeObjectPool() {/* empty */}

protected:
   /** Used to ensure thread safety.  */
   virtual void Lock() {_locker.Lock();}

   /** Used to ensure thread safety.  */
   virtual void Unlock() {_locker.Unlock();}

private:
   BLocker _locker;
};

#endif
