/* This file is Copyright 2001 Level Control Systems.  See the included LICENSE.txt file for details. */

#ifndef FILE_DATA_IO
#define FILE_DATA_IO

#include <stdio.h>
#include "dataio/PortableDataIO.h"

/**
 *  Data I/O to and from a stdio FILE. 
 */
class FileDataIO : public PortableDataIO
{
public:
   /** Constructor.
    *  @param file File to read from or write to.  Becomes property of this FileDataIO object,
    *         and will be fclose()'d when this object is deleted.
    */
   FileDataIO(FILE * file) : _file(file) {/* empty */}

   /** Destructor.
    *  Calls fclose() on the held file.
    */
   virtual ~FileDataIO() {fclose(_file);}

   /** Reads bytes from our file and places them into (buffer).
    *  @param buffer Buffer to write the bytes into
    *  @param size Number of bytes in the buffer.
    *  @return Number of bytes read, or -1 on error.  
    *  @see PortableDataIO::Read()
    */
   virtual ssize_t Read(void * buffer, size_t size)  
   {
      ssize_t ret = fread(buffer, 1, size, _file);
      return (ret > 0) ? ret : -1;  // EOF is an error, and it's returned as zero
   }

   /** Takes bytes from (buffer) and writes them out to our file.
    *  @param buffer Buffer to read the bytes from.
    *  @param size Number of bytes in the buffer.
    *  @return Number of bytes writte, or -1 on error.
    *  @see PortableDataIO::Write()
    */
   virtual ssize_t Write(const void * buffer, size_t size)
   {
      ssize_t ret = fwrite(buffer, 1, size, _file);
      return (ret > 0) ? ret : -1;   // zero is an error
   }

   /** Flushes the file output by calling fflush() */
   virtual void FlushOutput() {fflush(_file);}
   
private:
   FILE * _file;
};

#endif
