/* This file is Copyright 2001 Level Control Systems.  See the included LICENSE.txt file for details. */

#ifndef NULL_DATA_IO
#define NULL_DATA_IO

#include "dataio/PortableDataIO.h"

/**
 *  Data I/O equivalent to /dev/null.  
 */
class NullDataIO : public PortableDataIO
{
public:
   /** Default Constructor. */
   NullDataIO() {/* empty */}

   /** Virtual Destructor, to keep C++ honest */
   virtual ~NullDataIO() {/* empty */}

   /** 
    *  No-op method, always returns zero.
    *  @param buffer Points to a buffer to read bytes into (ignored).
    *  @param size Number of bytes in the buffer (ignored).
    *  @return zero.
    */
   virtual ssize_t Read(void * /*buffer*/, size_t /*size*/)  {return 0;}

   /** 
    *  No-op method, always returns (size).
    *  @param buffer Points to a buffer to write bytes from (ignored).
    *  @param size Number of bytes in the buffer (ignored).
    *  @return (size).
    */
   virtual ssize_t Write(const void * /*buffer*/, size_t size) {return size;}

   /** 
    *  No-op method.
    *  This method doesn't do anything at all.
    */
   virtual void FlushOutput() {/* empty */}
};

#endif
