/* This file is Copyright 2001 Level Control Systems.  See the included LICENSE.txt file for details. */

#ifndef PORTABLE_DATA_IO_H
#define PORTABLE_DATA_IO_H

#include "support/PortableStatusConstants.h"
#include "support/PortableTypeDefs.h"
#include "support/PortableMacros.h"

/** 
 *  Abstract base class for a byte-stream Data I/O interface, similar to Be's BDataIO.
 */
class PortableDataIO
{
public:
   /**
    *  Virtual destructor, to keep C++ honest.
    */
   virtual ~PortableDataIO() {}

   /** Tries to place (size) bytes of new data into (buffer).  Returns the
    *  actual number of bytes placed, or a negative value if there
    *  was an error.
    *  @param buffer Buffer to write the bytes into
    *  @param size Number of bytes in the buffer.
    *  @return Number of bytes read, or -1 on error.   
    */
   virtual ssize_t Read(void * buffer, size_t size) = 0;

   /** Takes (size) bytes from (buffer) and pushes them in to the
    *  outgoing I/O stream.  Returns the actual number of bytes 
    *  read from (buffer) and pushed, or a negative value if there
    *  was an error.
    *  @param buffer Buffer to read the bytes from.
    *  @param size Number of bytes in the buffer.
    *  @return Number of bytes writte, or -1 on error.        
    */
   virtual ssize_t Write(const void * buffer, size_t size) = 0;

   /**
    * Flushes the output buffer, if possible.  For some implementations,
    * this is a no-op.  For others (e.g. TCPSocketDataIO) this can be
    * called to improve latency of outgoing data blocks.
    */
   virtual void FlushOutput() = 0;
};

#endif
