/* This file is Copyright 2001 Level Control Systems.  See the included LICENSE.txt file for details. */

#ifndef PLAIN_TEXT_IO_GATEWAY
#define PLAIN_TEXT_IO_GATEWAY

#include "dataio/PortableDataIO.h"
#include "message/PortableMessage.h"
#include "refcount/PortableRefCount.h"
#include "iogateway/AbstractMessageIOGateway.h"

// This is the name of the string field used to hold text lines.
#define PR_NAME_TEXT_LINE "tl"

/** 
 * This gateway translates lines of text (separated by \r, \n, or \r\n) into
 * PortableMessages.  It can be used for "telnet-style" net interactions.
 * Incoming and outgoing messages may have one or more strings in their PR_NAME_TEXT_LINE field.
 * Each of these strings represents a line of text (separator chars not included)
 * class PlainTextMessageIOGateway : public AbstractMessageIOGateway
 */
{
public:
   /** This object becomes the owner of (io) and will delete it later. */
   PlainTextMessageIOGateway(PortableDataIO * io);

   /** Destructor */
   virtual ~PlainTextMessageIOGateway();

   virtual status_t DoOutput();
   virtual status_t DoInput();
   virtual bool HasBytesToOutput() const;

private:
   PortableMessageRef _currentSendingMessage;
   uint32 _currentSendLine;
   uint32 _currentSendOffset;

   PortableString _incomingText;
};

#endif
