/* This file is Copyright 2001 Level Control Systems.  See the included LICENSE.TXT file for details. */

package com.lcs.muscle.iogateway;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import com.lcs.muscle.support.UnflattenFormatException;
import com.lcs.muscle.message.PortableMessage;

/** Interface for an object that knows how to translate bytes
  * into PortableMessages, and vice versa.
  */
public interface AbstractMessageIOGateway {
    /** Reads from the input stream until a PortableMessage can be assembled and returned.
      * @param in The input stream to read from.
      * @throws IOException if there is an error reading from the stream.
      * @throws UnflattenFormatException if there is an error parsing the data in the stream.
      * @return The next assembled PortableMessage.
      */
    public PortableMessage unflattenMessage(DataInput in) throws IOException, UnflattenFormatException;
    
    /** Converts the given PortableMessage into bytes and sends it out the stream.
      * @param out the data stream to send the converted bytes to.
      * @param message the PortableMessage to convert.
      * @throws IOException if there is an error writing to the stream.
      */
    public void flattenMessage(DataOutput out, PortableMessage msg) throws IOException;
}
