/* This file is Copyright 2001 Level Control Systems.  See the included LICENSE.txt file for details. */

#ifndef NETWORK_UTILITY_FUNCTIONS_H
#define NETWORK_UTILITY_FUNCTIONS_H

#include "netutil/TimeUtilityFunctions.h"

/** Convenience method for connecting with TCP to a given hostName/port.
 * @param hostName The host name (or ASCII IP address) of the computer to connect to.
 * @param port The port number to connect to.
 * @param debugTitle If non-NULL, debug output to stdout will be enabled and debug output will be prefaced by this string.
 * @param debugOutputOnErrorsOnly if true, debug output will be printed only if an error condition occurs.
 * @return the sockfd if the connection is successful (greater than or equal to 0), -1 if the connection attempt failed.
 */
int Connect(const char * hostName, int port, const char * debugTitle = NULL, bool debugOutputOnErrorsOnly = true);

/** Returns a string representation of the IP address (address).  Useful on OS's where inet_ntoa isn't available.
 *  @param address The 4-byte IP address to translate into text.
 *  @param outBuf The ASCII representation of the IP address will be written into this buffer,
 *                which must be at least 13 bytes long.
 */
void Inet_NtoA(uint32 address, char * outBuf);

/** Fills out (retHostIP) and (retPort) with information about the remote peer on the other end of (socket).  
 *  @param socket The socket descriptor to find out info about.
 *  @param retHostIP On success, the IP address of the computer that (socket) is connected to
 *                   will be written into this object.
 *  @param retPort On success, the remote port that (socket) is connected to will be written into this object.
 *  @return B_NO_ERROR on success, B_ERROR on failure (such as if the socket isn't valid and connected).
 */
status_t GetPeerInfo(int socket, uint32 & retHostIP, uint16 & retPort);

/** Calls fork(), setsid(), chdir(), umask(), etc, to fork an independent daemon process.
 *  Also closes all open file descriptors.
 *  Note that this function will call exit() on the parent process if successful,
 *  and thus won't ever return in that process. 
 *  @return B_NO_ERROR on success (the child process will see this), B_ERROR on failure.
 */
status_t BecomeDaemonProcess();

#endif
