/* This file is Copyright 2000 Level Control Systems.  See the included LICENSE.txt file for details. */  

#include <stdio.h>
#include "string/PortableString.h"
#include "refcount/PortableRefCount.h"
#include "queue/PortableQueue.h"
#include "hashtable/PortableHashtable.h"

class TestClass
{
public:
   TestClass() : _name("Default") {printf("Constructing [%s]\n",_name.Cstr());}
   TestClass(const char * name) : _name(name) {printf("Constructing [%s]\n",_name.Cstr());}
   ~TestClass() {printf("Destroying [%s]\n",_name.Cstr());}

   const char * GetName() const {return _name.Cstr();}

private:
   PortableString _name;
};

// This program exercises the PortableRef class.
void main(void) 
{
   PortableRef<const TestClass>::RefPool trpool;
   PortableRef<const TestClass>::ItemPool tpool;
 
   {
      PortableRef<const TestClass> * refPtr = NULL;
      PortableRef<const TestClass> extraRef;
      {
         printf("1\n");
         PortableRef<const TestClass> testRef(new TestClass("Test"), &tpool, &trpool);
         printf("2\n");
         extraRef = testRef;
         printf("3\n");
         refPtr = new PortableRef<const TestClass>(extraRef);
         printf("4\n");

         PortableRef<const TestClass> copy(*refPtr);
         printf("5\n");
      }
      printf("6\n");
      delete refPtr;
      //printf("7 [%s]\n", extraRef()->GetName());
   }

   {
      printf("Checking queue...\n");
      PortableQueue<PortableRef<TestClass> > q;
      printf("Adding refs...\n");
      for (int i=0; i<10; i++) 
      {
         char temp[50]; sprintf(temp, "%i", i);
         q.AddTail(PortableRef<TestClass>(new TestClass(temp), NULL, NULL));
      }
      printf("Removing refs...\n");
      while(q.CountItems() > 0) q.RemoveHead();
      printf("Done with queue test!\n");
   }

   {
      printf("Checking hashtable\n");
      PortableHashtable<PortableString, PortableRef<TestClass> > ht(PortableStringHashFunc);
      printf("Adding refs...\n");
      for (int i=0; i<10; i++) 
      {
         char temp[50]; sprintf(temp, "%i", i);
         ht.Put(PortableString(temp), PortableRef<TestClass>(new TestClass(temp), NULL, NULL));
      }
      printf("Removing refs...\n");
      for (int i=0; i<10; i++) 
      {
         char temp[50]; sprintf(temp, "%i", i);
         ht.Remove(PortableString(temp));
      }
      printf("Done with hash table test!\n");
   }
    
   printf("Checking for memory leaks... watch free mem in about box, press CTRL-C when done\n");
   while(1)
   {
      PortableRef<const TestClass> r2(tpool.GetObject(), &tpool, &trpool);
   }
}
